﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FindFacesSamplerRotation
{
    public partial class Form1 : Form
    {
        public class Detection
        {
            public int Index;
            public Point Position;
            public float Angle;
            public double Distance;

            public Detection( int AIndex, Point APosition, float AAngle, double ADistance )
            {
                Index = AIndex;
                Position = APosition;
                Angle = AAngle;
                Distance = ADistance;
            }
        }

        private System.Collections.Generic.List<Detection> FDetections = new System.Collections.Generic.List<Detection>();
        private bool FRunning = true;

        public Form1()
        {
            InitializeComponent();
            radialBasisFunctionNetwork1.LoadFromFile("D:\\RBFFaceTraining2.ctd");
        }

        private void genericFilter1_ProcessData(object Sender, Mitov.VideoLab.ProcessVideoNotifyArgs Args)
        {
            if (!FRunning)
                return;

            FRunning = false;

            Args.InBuffer.SetFormat(Mitov.VideoLab.VideoFormat.GrayScale);
            genericFilter2.SendStartCommand((uint)Args.InBuffer.GetWidth(), (uint)Args.InBuffer.GetHeight(), 1000);
            genericFilter2.SendData(Args.InBuffer);
            genericFilter2.SendStopCommand();

            System.Drawing.Font AFont = new System.Drawing.Font("Arial", 10);

            System.Drawing.Bitmap aBitmap = Args.InBuffer.ToBitmap();
            System.Drawing.Graphics AGraphics = System.Drawing.Graphics.FromImage(aBitmap);

            FDetections.ForEach(
                    delegate(Detection ADetection)
                    {
                        AGraphics.ResetTransform();
                        AGraphics.TranslateTransform(ADetection.Position.X, ADetection.Position.Y);
                        AGraphics.RotateTransform(ADetection.Angle);
                        AGraphics.TranslateTransform(-ADetection.Position.X, -ADetection.Position.Y);
                        AGraphics.DrawRectangle(System.Drawing.Pens.Red, ADetection.Position.X - 16, ADetection.Position.Y - 16, 32, 32);
                        AGraphics.DrawString(ADetection.Index.ToString(), AFont, System.Drawing.Brushes.Red, ADetection.Position);
                    }
                );

            imageDisplay1.DisplayBitmap(aBitmap);
        }

        private void genericFilter3_ProcessData(object Sender, Mitov.VideoLab.ProcessVideoNotifyArgs Args)
        {
            Mitov.SignalLab.RealBuffer AFeaturesBuffer = new Mitov.SignalLab.RealBuffer( 32 * 32 );
            Args.InBuffer.SetFormat( Mitov.VideoLab.VideoFormat.GrayScale );
            for( int i = 0; i < 32 * 32; ++ i )
                AFeaturesBuffer[ i ] = ((double)Args.InBuffer.GetGrayScale( i % 32, i / 32 ))/ 256;

            radialBasisFunctionNetwork1.Predict( AFeaturesBuffer );
        }

        private void radialBasisFunctionNetwork1_Result(object Sender, Mitov.IntelligenceLab.RadialBasisFunctionResultEventArgs Args)
        {
            if( Args.Result.Count == 0 )
                return;

            int ACategory = (int)System.Math.Round( Args.Result.get_Items( 0 ).Neuron.Category );

            bool ADontAdd = false;
            Point ADetectedPosition = sampler1.CurrectPosition;
            float ADetectedAngle = sampler1.CurrectAngle;
            double ADistance = Args.Result.get_Items( 0 ).Distance;
            FDetections.RemoveAll( 
                    delegate( Detection AItem )
                    {
                        if( ADetectedPosition != AItem.Position )
                            return( false );

                        if( ACategory != AItem.Index )
                            return( false );

                        if( ADistance < AItem.Distance )
                            return( true );

                        ADontAdd = true;
                        return( false );
                    }
                );

            if( ADontAdd )
                return;

            FDetections.Add(new Detection(ACategory, ADetectedPosition, ADetectedAngle, ADistance));
        }

    }
}
