﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SimpleOCR
{
    public partial class Form1 : Form
    {
        private const int IMAGE_WIDTH = 28;
        private const int IMAGE_HEIGHT = 28;

        private System.Collections.Generic.List<System.Drawing.Bitmap> FTrainingBitmaps = new System.Collections.Generic.List<System.Drawing.Bitmap>();
        private System.Collections.Generic.List<System.Drawing.Bitmap> FTestBitmaps = new System.Collections.Generic.List<System.Drawing.Bitmap>();

        private Mitov.SignalLab.RealBuffer FTrainingLabels;

        private Mitov.SignalLab.RealBuffer FTestLabels;
        private int FResult = -1;

        public Form1()
        {
            InitializeComponent();

            PopulateImageList(trainingImageList, FTrainingBitmaps, @"C:\Conferences\CodeRage XI\AI\Sources\Data\OCR\trainingimages");
            FTrainingLabels = LoadLabels(@"C:\Conferences\CodeRage XI\AI\Sources\Data\OCR\traininglabels");
            PopulateListView(trainingListView, FTrainingLabels);

            PopulateImageList(testImageList, FTestBitmaps, @"C:\Conferences\CodeRage XI\AI\Sources\Data\OCR\testimages");
            FTestLabels = LoadLabels(@"C:\Conferences\CodeRage XI\AI\Sources\Data\OCR\testlabels");
            PopulateListView(testListView, FTestLabels);

        }

        private Mitov.SignalLab.RealBuffer ExtractImageFeatures(System.Drawing.Bitmap ABitmep)
        {
            Mitov.SignalLab.RealBuffer AResult = new Mitov.SignalLab.RealBuffer(ABitmep.Width * ABitmep.Height);

            for (int y = 0; y < IMAGE_HEIGHT; ++y)
                for (int x = 0; x < IMAGE_WIDTH; ++x)
                {
                    System.Drawing.Color AColor = ABitmep.GetPixel(x, y);
                    if (AColor.ToArgb() == System.Drawing.Color.White.ToArgb())
                        AResult[y * IMAGE_WIDTH + x] = 0;

                    else
                        AResult[y * IMAGE_WIDTH + x] = 1;
//                    double aValue = (ABitmep.GetPixel(x, y) != System.Drawing.Color.White) ? 1 : 0;
//                    AResult[y * IMAGE_WIDTH + x] = (ABitmep.GetPixel(x, y) != System.Drawing.Color.White) ? 1 : 0;
                }

            return AResult;
        }
/*
        private Mitov.SignalLab.RealMatrixBuffer ExtractAllFeatures(System.Collections.Generic.List<System.Drawing.Bitmap> ABitmapList )
        {
            Mitov.SignalLab.RealMatrixBuffer AResult = new Mitov.SignalLab.RealMatrixBuffer(ABitmapList.Count, IMAGE_WIDTH * IMAGE_HEIGHT);

            for (int i = 0; i < ABitmapList.Count; ++i)
                for( int y = 0; y < IMAGE_HEIGHT; ++y )
                    for( int x = 0; x < IMAGE_WIDTH; ++x )
                        AResult[i][y * IMAGE_WIDTH + x] = (ABitmapList[i].GetPixel(x, y) != System.Drawing.Color.White) ? 1 : 0;


            return AResult;
        }
*/
        private System.Collections.Generic.List<Mitov.SignalLab.RealBuffer> ExtractAllFeatures(System.Collections.Generic.List<System.Drawing.Bitmap> ABitmapList)
        {
            System.Collections.Generic.List<Mitov.SignalLab.RealBuffer> AResult = new System.Collections.Generic.List<Mitov.SignalLab.RealBuffer>();

            for (int i = 0; i < ABitmapList.Count; ++i)
                AResult.Add(ExtractImageFeatures(ABitmapList[i]));

            return AResult;
        }

        private void PopulateListView(System.Windows.Forms.ListView AListView, Mitov.SignalLab.RealBuffer ALabels)
        {
            AListView.BeginUpdate();
            for( int i = 0; i < ALabels.Size; ++ i)
            {
                ListViewItem AItem = AListView.Items.Add( ((int)ALabels[ i ]).ToString() );
                AItem.ImageIndex = i;
            }

            AListView.EndUpdate();
        }

        private Mitov.SignalLab.RealBuffer LoadLabels( String AFileName )
        {
            string[] ALines = System.IO.File.ReadAllLines(AFileName);
            Mitov.SignalLab.RealBuffer AResult = new Mitov.SignalLab.RealBuffer(ALines.Length);

            for (int i = 0; i < ALines.Length; ++i)
            {
                int ANumber;
                Int32.TryParse( ALines[ i ], out ANumber );
                AResult[i] = ANumber;
            }

            return AResult;
        }

        private void PopulateImageList( System.Windows.Forms.ImageList AImageList, System.Collections.Generic.List<System.Drawing.Bitmap> ABitmapList, String AFile )
        {
            System.Drawing.Bitmap ABitmep = new System.Drawing.Bitmap(IMAGE_WIDTH, IMAGE_HEIGHT, System.Drawing.Imaging.PixelFormat.Format24bppRgb);

            string[] ALines = System.IO.File.ReadAllLines(AFile);

            for (int i = 0; i < ALines.Length; ++i)
            {
                if (((i % IMAGE_HEIGHT) == 0) && (i > 0))
                {
                    AImageList.Images.Add(ABitmep);
                    ABitmapList.Add(ABitmep);
                    ABitmep = new System.Drawing.Bitmap(IMAGE_WIDTH, IMAGE_HEIGHT, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
                }

                for( int x = 0; x < 28; ++x )
                {
                    switch( ALines[ i ][ x ] )
                    {
                        case '+' :
                            ABitmep.SetPixel( x, i % IMAGE_HEIGHT, System.Drawing.Color.Gray );
                            break;

                        case '#' :
                            ABitmep.SetPixel( x, i % IMAGE_HEIGHT, System.Drawing.Color.Black );
                            break;

                        default:
                            ABitmep.SetPixel( x, i % IMAGE_HEIGHT, System.Drawing.Color.White );
                            break;
                    }
                }
            }

            AImageList.Images.Add(ABitmep);
            ABitmapList.Add(ABitmep);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            System.Collections.Generic.List<Mitov.SignalLab.RealBuffer> ATrainingData = ExtractAllFeatures(FTrainingBitmaps);
            for (int i = 0; i < ATrainingData.Count; ++i)
                radialBasisFunctionNetwork1.Train(ATrainingData[ i ], FTrainingLabels[ i ]);

            System.Collections.Generic.List<Mitov.SignalLab.RealBuffer> ATestData = ExtractAllFeatures(FTestBitmaps);

            Application.DoEvents();

            testListView.BeginUpdate();
            int AErrorsCount = 0;
            for (int i = 0; i < ATestData.Count; ++i)
            {
                radialBasisFunctionNetwork1.Predict(ATestData[i]);
                if( FResult != FTestLabels[ i ] )
                {
                    testListView.Items[ i ].Text = "Error " + FResult.ToString() + "(" + ((int)FTestLabels[ i ] ).ToString() + ")";
                    ++AErrorsCount;
                }

                else
                    testListView.Items[ i ].Text = FResult.ToString() + "(" + ((int)FTestLabels[ i ] ).ToString() + ")";

            }

            testListView.EndUpdate();
            label1.Text = AErrorsCount.ToString() + " Errors of " + testListView.Items.Count.ToString() + " - " + (((float)testListView.Items.Count - AErrorsCount) * 100 / testListView.Items.Count).ToString() + "% correct";
        }

        private void radialBasisFunctionNetwork1_Result(object Sender, Mitov.IntelligenceLab.RadialBasisFunctionResultEventArgs Args)
        {
            if( Args.Result.Count == 0 )
            {
                FResult = -1;
                return;
            }

            FResult = (int)Args.Result.get_Items(0).Neuron.Category;
        }
    }
}
